@@ -24,7 +24,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: PostReader.h 540 2005-08-31 11:27:51Z svn $
+ * $Id: PostReader.h 556 2005-09-01 13:54:29Z svn $
  *****************************************************************************/
 
 #ifndef POSTREADER_H
@@ -45,7 +45,7 @@ class PostReader
 {
 public:
     PostReader(apr_size_t block_size);
-    virtual void read(char *buffer, apr_size_t *size) = 0;
+    virtual void read(char *buffer, apr_size_t size, apr_size_t *read_size) = 0;
     virtual ~PostReader();
 protected:
     apr_size_t get_block_size();
