@@ -24,7 +24,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploadItemList.h 479 2005-07-23 12:53:27Z svn $
+ * $Id: UploadItemList.h 550 2005-09-01 07:23:16Z svn $
  *****************************************************************************/
 
 #ifndef UPLOADITEMLIST_H
@@ -60,11 +60,15 @@ public:
     UploadItemList(apr_pool_t *pool,
                    const char *file_dir, const char *thumb_dir,
                    apr_size_t total_number_limit, apr_uint64_t total_size_limit);
-    void add(const char *file_path, bool is_make_thumb=false);
+    void add(const char *file_path, bool is_new=false);
+    void add(item_info *info);
     apr_size_t size() const;
-    apr_size_t thumb_size() const;
-    apr_uint64_t total_file_size() const;
+    bool empty() const;
+    apr_uint64_t get_total_file_size() const;
+    apr_size_t get_thumb_size() const;
     apr_time_t get_mtime() const;
+    void set_mtime();
+    bool is_need_update() const;
     void remove(const char *file_path, const char *password);
     TemplateExecutor::variable *to_varray(apr_pool_t *pool,
                                           apr_size_t start,
@@ -82,6 +86,11 @@ public:
     static void dump_list(UploadItemList *item_list);
 #endif
 private:
+    static const char PATH_CACHE_FILENAME[];
+    static const char HEADER_CACHE_FILENAME[];
+    static const char TMP_SUFFIX[];
+    static const char PATH_CACHE_DELIMITER;
+
     struct ItemCompare
     {
         bool operator()(const item_info *lhs, const item_info *rhs) const
@@ -103,15 +112,8 @@ private:
     typedef list<string> thumb_list;
     typedef thumb_list::const_iterator thumb_iterator;
 
-    UploadItemList(const UploadItemList&);
-    UploadItemList &operator=(const UploadItemList&);
-
-    void init();
-    item_iterator begin() const;
-    item_iterator end() const;
-    void remove(item_info *info);
-
     apr_pool_t *pool_;
+    apr_pool_t *data_pool_;
     const char *file_dir_;
     const char *thumb_dir_;
     const apr_size_t total_number_limit_;
@@ -119,9 +121,41 @@ private:
 
     UploadItemReader reader;
     apr_uint64_t total_size_;
-    apr_time_t mtime_;
     item_list ilist_;
     thumb_list tlist_;
+    apr_time_t mtime_;
+
+    UploadItemList(const UploadItemList&);
+    UploadItemList &operator=(const UploadItemList&);
+
+    void init();
+    bool load_file();
+    bool load_file_by_any();
+    void load_file_by_item();
+    void load_file_by_cache();
+    void load_thumb();
+    void add_new(item_info *info);
+    item_iterator begin() const;
+    item_iterator end() const;
+    void remove(item_info *info);
+    void sort();
+
+    const char *create_path(apr_pool_t *tmp_pool, const char *name);
+    const char *create_tmp_path(apr_pool_t *tmp_pool, const char *name,
+                                apr_file_t **tmp_file);
+
+    const char *get_path_cache(apr_pool_t *tmp_pool);
+    const char *get_header_cache(apr_pool_t *tmp_pool);
+    const char *get_tmp_path_cache(apr_pool_t *tmp_pool, apr_file_t **cache_file);
+    const char *get_tmp_header_cache(apr_pool_t *tmp_pool, apr_file_t **cache_file);
+
+    void read_cache(apr_pool_t *tmp_pool,
+                    char **paths, UploadItem::header **headers, apr_size_t *number);
+    void read_path_cache(apr_pool_t *tmp_pool, char **paths, apr_size_t *number);
+    void read_header_cache(apr_pool_t *tmp_pool, UploadItem::header **headers, apr_size_t *number);
+    void write_cache();
+    void write_path_cache(apr_pool_t *tmp_pool);
+    void write_header_cache(apr_pool_t *tmp_pool);
 };
 
 #endif
