@@ -24,7 +24,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: ApachePostReader.cpp 550 2005-09-01 07:23:16Z svn $
+ * $Id: ApachePostReader.cpp 557 2005-09-01 14:15:15Z svn $
  *****************************************************************************/
 
 #include "ApachePostReader.h"
@@ -34,13 +34,11 @@
 #define APR_WANT_MEMFUNC
 #include "apr_want.h"
 
-// #ifdef DEBUG
+#ifdef DEBUG
 #include <iostream>
-// #endif
+#endif
 
 
-#include "http_protocol.h"
-
 /******************************************************************************
  * public ᥽å
  *****************************************************************************/
@@ -54,14 +52,12 @@ ApachePostReader::ApachePostReader(apr_size_t block_size, request_rec *r)
 
 }
 
-void ApachePostReader::read(char *buffer, apr_size_t *size)
+
+void ApachePostReader::read(char *buffer, apr_size_t size, apr_size_t *read_size)
 {
     const char *read_data;
 
-    ap_rprintf(request_, "CALL: read\n");
-    cout << "CALL: read" << endl;
-
-    *size = 0;
+    *read_size = 0;
 
     if (UNLIKELY(is_eos_)) {
         return;
@@ -79,12 +75,15 @@ void ApachePostReader::read(char *buffer, apr_size_t *size)
         return;
     } else if (APR_BUCKET_IS_FLUSH(bucket_)) {
         bucket_ = APR_BUCKET_NEXT(bucket_);
-        return read(buffer, size);
+        return read(buffer, size, read_size);
     }
 
-    apr_bucket_read(bucket_, &read_data, size, APR_BLOCK_READ);
+    apr_bucket_read(bucket_, &read_data, read_size, APR_BLOCK_READ);
+    memmove(buffer, read_data, *read_size);
 
     bucket_ = APR_BUCKET_NEXT(bucket_);
+
+    return;
 }
 
 
@@ -93,86 +92,14 @@ void ApachePostReader::read(char *buffer, apr_size_t *size)
  *****************************************************************************/
 void ApachePostReader::get_brigade()
 {
-    ap_rprintf(request_, "CALL: get_brigade\n");
-    cout << "CALL: get_brigade" << endl;
-
-    if (ap_get_brigade(request_->input_filters, brigade_, AP_MODE_READBYTES,
-                       APR_BLOCK_READ, get_block_size()) != APR_SUCCESS) {
+    if (UNLIKELY(ap_get_brigade(request_->input_filters, brigade_, AP_MODE_READBYTES,
+                                APR_BLOCK_READ, get_block_size()) != APR_SUCCESS)) {
         throw "ꥯɤ߹ߤν˼Ԥޤ";
     }
 
     bucket_ = APR_BRIGADE_FIRST(brigade_);
 }
 
-
-/******************************************************************************
- * ƥ
- *****************************************************************************/
-#ifdef DEBUG_ApachePostReader
-#include "httpd.h"
-#include "http_config.h"
-#include "http_protocol.h"
-#include "ap_config.h"
-
-static const apr_size_t BLOCK_SIZE  = 4096;
-
-extern "C" {
-static int test_handler(request_rec *r)
-{
-    ApachePostReader reader(BLOCK_SIZE, r);
-
-    if (strcmp(r->handler, "my_test")) {
-        return DECLINED;
-    }
-
-    r->content_type = "text/plain";
-
-    ap_rputs("mod_test.....\n", r);
-
-    if (r->header_only) {
-        return OK;
-    }
-
-    ap_rputs("begin!\n", r);
-
-//     try {
-//         char buffer[BLOCK_SIZE];
-//         apr_size_t size;
-
-//         while (true) {
-//             reader.read(buffer, &size);
-
-//             ap_rprintf(r, "read: %d\n", size);
-
-//             if (size == 0) {
-//                 break;
-//             }
-//         }
-//     } catch(const char *message) {
-//         ap_rprintf(r, "Error: %s\n", message);
-
-//         return HTTP_INTERNAL_SERVER_ERROR;
-//     }
-
-    ap_rputs("finish!\n", r);
-
-    return OK;
-}
-
-static void test_register_hooks(apr_pool_t *p)
-{
-    ap_hook_handler(test_handler, NULL, NULL, APR_HOOK_MIDDLE);
-}
-
-module AP_MODULE_DECLARE_DATA test_module = {
-    STANDARD20_MODULE_STUFF,
-    NULL, NULL, NULL, NULL, NULL,
-    test_register_hooks
-};
-}
-
-#endif
-
 // Local Variables:
 // mode: c++
 // buffer-file-coding-system: euc-japan-dos
