@@ -24,11 +24,12 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploadItem.cpp 479 2005-07-23 12:53:27Z svn $
+ * $Id: UploadItem.cpp 535 2005-08-30 20:15:17Z svn $
  *****************************************************************************/
 
 #include "UploadItem.h"
 #include "Auxiliary.h"
+#include "Misc.h"
 
 #include "apr_strings.h"
 #include "apr_time.h"
@@ -139,7 +140,7 @@ const char *UploadItem::get_time() const
     char *time_str;
     apr_size_t size;
 
-    time_str = reinterpret_cast<char *>(apr_palloc(pool_, sizeof(char)*MAX_TIME_LENGTH));
+    APR_PALLOC(time_str, char *, pool_, sizeof(char)*MAX_TIME_LENGTH);
 
     if (time_str == NULL) {
         throw "ɽΤΥݤ˼Ԥޤ";
@@ -157,7 +158,7 @@ void UploadItem::dump_header(header *header)
     cerr << "identifier: "      << header->identifier       << endl;
     cerr << "version: "         << header->version          << endl;
     cerr << "time: "            << header->time             << endl;
-    cerr << "file_size: "       << reinterpret_cast<unsigned int>(header->file_size) << endl;
+    cerr << "file_size: "       << static_cast<unsigned int>(header->file_size) << endl;
     cerr << "file_name: "       << header->file_name        << endl;
     cerr << "file_mime: "       << header->file_mime        << endl;
     cerr << "remove_pass: "     << header->remove_pass      << endl;
