@@ -24,11 +24,12 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploadItemCreator.cpp 468 2005-07-18 10:11:42Z svn $
+ * $Id: UploadItemCreator.cpp 535 2005-08-30 20:15:17Z svn $
  *****************************************************************************/
 
 #include "UploadItemCreator.h"
 #include "CharCodeConverter.h"
+#include "Misc.h"
 
 
 /******************************************************************************
@@ -40,10 +41,9 @@ UploadItem::header *UploadItemCreator::create_header(apr_pool_t *pool,
 {
     UploadItem::header *data;
 
-    validate_query(qmap);
+    validate_query(pool, qmap);
 
-    data = reinterpret_cast<UploadItem::header *>(apr_pcalloc(pool,
-                                                         sizeof(UploadItem::header)));
+    APR_PCALLOC(data, UploadItem::header *, pool, sizeof(UploadItem::header));
 
     strncpy(data->identifier, PACKAGE_NAME, sizeof(PACKAGE_NAME));
     strncpy(data->version, PACKAGE_VERSION, sizeof(PACKAGE_VERSION));
@@ -51,15 +51,15 @@ UploadItem::header *UploadItemCreator::create_header(apr_pool_t *pool,
     data->time = apr_time_now();
     strncpy(data->comment,
             ap_escape_html(pool, (*qmap)[UploadItem::COMMENT_PARAM].text.c_str()),
-            UploadItem::MAX_COMMENT_LENGTH);
+            UploadItem::MAX_COMMENT_LENGTH-1);
 
     // ѥɤ ap_escape_html ʤ
     strncpy(data->download_pass, (*qmap)[UploadItem::DOWNLOAD_PASS_PARAM].text.c_str(),
-            UploadItem::MAX_DOWNLOAD_PASS_LENGTH);
+            UploadItem::MAX_DOWNLOAD_PASS_LENGTH-1);
 
     // ѥɤ ap_escape_html ʤ
     strncpy(data->remove_pass, (*qmap)[UploadItem::REMOVE_PASS_PARAM].text.c_str(),
-            UploadItem::MAX_REMOVE_PASS_LENGTH);
+            UploadItem::MAX_REMOVE_PASS_LENGTH-1);
 
     set_file_data(pool, data, (*qmap)[UploadItem::FILE_PARAM].file, qmap);
 
@@ -72,38 +72,47 @@ UploadItem::header *UploadItemCreator::create_header(apr_pool_t *pool,
 /******************************************************************************
  * private ᥽å
  *****************************************************************************/
-void UploadItemCreator::validate_query(UploadParser::query_map *qmap)
+void UploadItemCreator::validate_query(apr_pool_t *pool,
+                                       UploadParser::query_map *qmap)
 {
+    const unsigned char *code_pat;
+    const char *file_name;
+
     // MEMO:
     // - ե륵 0 ξϡ顼Ф
     // - ȤѥɤˤĤƤϡȤǤ⥨顼Фʤ
 
+    code_pat = reinterpret_cast<const unsigned char *>((*qmap)[UploadItem::CODE_PAT_PARAM].text.c_str());
+
     if (qmap->count(UploadItem::FILE_PARAM) != 1) {
         throw "եλ꤬Ǥ";
-    } else if (((*qmap)[UploadItem::FILE_PARAM].type != UploadParser::FILE) ||
+    }
+    convert_code(pool, (*qmap)[UploadItem::FILE_PARAM].text, code_pat);
+    if (((*qmap)[UploadItem::FILE_PARAM].type != UploadParser::FILE) ||
                ((*qmap)[UploadItem::FILE_PARAM].file.size == 0)) {
         throw "ե뤬Ǥ";
     } else if ((*qmap)[UploadItem::FILE_PARAM].file.mime.length() >= UploadItem::MAX_FILE_MIME_LENGTH) {
         throw "ե MIME Type Ĺޤ";
     }
-    {
-        const char *file_name = (*qmap)[UploadItem::FILE_PARAM].file.name.c_str();
 
-        if (strlen(file_name) >= UploadItem::MAX_FILE_NAME_LENGTH) {
-            throw "ե̾Ĺޤ";
-        }
-        do {
-            if ((*file_name == '"') || (*file_name == '\'') ||
-                (*file_name == '\r') || (*file_name == '\n') ||
-                (*file_name == '\\')) {
-                throw "ե̾Ǥ";
-            }
-        } while (*(++file_name) != '\0');
+    file_name = (*qmap)[UploadItem::FILE_PARAM].file.name.c_str();
+
+    if (strlen(file_name) >= UploadItem::MAX_FILE_NAME_LENGTH) {
+        throw "ե̾Ĺޤ";
     }
+    do {
+        if ((*file_name == '"') || (*file_name == '\'') ||
+            (*file_name == '\r') || (*file_name == '\n') ||
+            (*file_name == '\\')) {
+            throw "ե̾Ǥ";
+        }
+    } while (*(++file_name) != '\0');
 
     if (qmap->count(UploadItem::COMMENT_PARAM) != 1) {
         throw "Ȥλ꤬Ǥ";
-    } else if (((*qmap)[UploadItem::COMMENT_PARAM].type != UploadParser::TEXT) ||
+    }
+    convert_code(pool, (*qmap)[UploadItem::COMMENT_PARAM].text, code_pat);
+    if (((*qmap)[UploadItem::COMMENT_PARAM].type != UploadParser::TEXT) ||
                ((*qmap)[UploadItem::COMMENT_PARAM].text.length() >= UploadItem::MAX_COMMENT_LENGTH)) {
         throw "ȤĹޤ";
     }
@@ -127,6 +136,14 @@ void UploadItemCreator::validate_query(UploadParser::query_map *qmap)
     }
 }
 
+void UploadItemCreator::convert_code(apr_pool_t *pool, string &str,
+                                     const unsigned char *code_pat)
+{
+    str.assign(ap_escape_html(pool,
+                              CharCodeConverter::convert(pool, str.c_str(),
+                                                         code_pat)));
+}
+
 void UploadItemCreator::set_file_data(apr_pool_t *pool,
                                       UploadItem::header *data,
                                       UploadParser::file_content& file,
@@ -135,13 +152,8 @@ void UploadItemCreator::set_file_data(apr_pool_t *pool,
     data->file_size = file.size;
 
     strncpy(data->file_mime, ap_escape_html(pool, file.mime.c_str()),
-            UploadItem::MAX_FILE_MIME_LENGTH);
-
-    strncpy(data->file_name,
-            ap_escape_html(pool,
-                           CharCodeConverter::convert(pool, file.name.c_str(),
-                                                      (*qmap)[UploadItem::CODE_PAT_PARAM].text.c_str())),
-            UploadItem::MAX_FILE_NAME_LENGTH);
+            UploadItem::MAX_FILE_MIME_LENGTH-1);
+    strncpy(data->file_name, file.name.c_str(), UploadItem::MAX_FILE_NAME_LENGTH-1);
 }
 
 // Local Variables:
