@@ -24,7 +24,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: mod_uploader.cpp 512 2005-08-25 07:00:14Z svn $
+ * $Id: mod_uploader.cpp 554 2005-09-01 11:49:36Z svn $
  *****************************************************************************/
 
 #include "mod_uploader.h"
@@ -37,23 +37,28 @@
 #include "ThumbnailWriter.h"
 #endif
 #include "Auxiliary.h"
+#include "Misc.h"
 
 #include "http_core.h"
+#define APR_WANT_STRFUNC
+#include "apr_want.h"
 
 #include <stdlib.h>
-#include <string.h>
 
 #include <memory>
 
 #define MATCH(str, pat) (strstr(str, pat) != 0)
 
-#undef ERROR // for Visual C++
 #ifdef DEBUG
-#define ERROR(format, str) \
+#define SERROR(format, str) \
+    fprintf(stderr, format, str); fputc('\n', stderr)
+#define RERROR(format, str)                             \
     fprintf(stderr, format, str); fputc('\n', stderr)
 #else
-#define ERROR(format, str) \
-    ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, "[mod_uploader] " format, str)
+#define SERROR(format, str) \
+    ap_log_error(APLOG_MARK, APLOG_CRIT, 0, s, "[mod_uploader] " format, str)
+#define RERROR(format, str)                                             \
+    ap_log_rerror(APLOG_MARK, APLOG_CRIT, 0, r, "[mod_uploader] " format, str)
 #endif
 
 static int command_handler(request_rec *r, uconfig *config, const char *arg);
@@ -124,13 +129,13 @@ int view(request_rec *r, uconfig *config, const char *arg)
     ap_set_etag(r);
 
     if ((status = ap_meets_conditions(r)) != OK) {
-        config->unlock();
+        config->read_unlock();
         return status;
     }
 
     ap_set_content_type(r, "text/html");
     if (r->header_only) {
-        config->unlock();
+        config->read_unlock();
         return OK;
     }
 
@@ -141,11 +146,11 @@ int view(request_rec *r, uconfig *config, const char *arg)
 
         config->texecutor->exec(r->pool, r, tmpl->node, tmpl->imap, vmap.get());
 
-        config->unlock();
+        config->read_unlock();
 
         return OK;
     } catch(const char *message) {
-        config->unlock();
+        config->read_unlock();
 
         ap_rputs("<div class=\"warning\">", r);
         ap_rputs(message, r);
@@ -167,13 +172,13 @@ int thumbnail(request_rec *r, uconfig *config, const char *arg)
     ap_set_etag(r);
 
     if ((status = ap_meets_conditions(r)) != OK) {
-        config->unlock();
+        config->read_unlock();
         return status;
     }
 
     ap_set_content_type(r, "text/html");
     if (r->header_only) {
-        config->unlock();
+        config->read_unlock();
         return OK;
     }
 
@@ -184,11 +189,11 @@ int thumbnail(request_rec *r, uconfig *config, const char *arg)
 
         config->texecutor->exec(r->pool, r, tmpl->node, tmpl->imap, vmap.get());
 
-        config->unlock();
+        config->read_unlock();
 
         return OK;
     } catch(const char *message) {
-        config->unlock();
+        config->read_unlock();
 
         ap_rputs("<div class=\"warning\">", r);
         ap_rputs(message, r);
@@ -210,13 +215,13 @@ int rss(request_rec *r, uconfig *config, const char *arg)
     ap_set_etag(r);
 
     if ((status = ap_meets_conditions(r)) != OK) {
-        config->unlock();
+        config->read_unlock();
         return status;
     }
 
     ap_set_content_type(r, RssView::content_type);
     if (r->header_only) {
-        config->unlock();
+        config->read_unlock();
         return OK;
     }
 
@@ -226,11 +231,11 @@ int rss(request_rec *r, uconfig *config, const char *arg)
         view.exec(r->pool, r, config->url, config->item_list,
                   config->per_page_item_number);
 
-        config->unlock();
+        config->read_unlock();
 
         return OK;
     } catch(const char *message) {
-        config->unlock();
+        config->read_unlock();
 
         ap_rputs("<div class=\"warning\">", r);
         ap_rputs(message, r);
@@ -276,9 +281,9 @@ int upload(request_rec *r, uconfig *config, const char *arg)
         try {
             config->write_lock();
             config->item_list->add(file_name, true);
-            config->unlock();
+            config->write_unlock();
         } catch(const char *) {
-            config->unlock();
+            config->read_unlock();
             throw;
         }
 
@@ -359,7 +364,7 @@ int download(request_rec *r, uconfig *config, const char *arg)
                        apr_psprintf(r->pool,
                                     "%s; filename=\"%s\"", open_mode, file_name));
 
-        ap_send_fd(file, r, UploadItem::ITEM_HEADER_SIZE, header->file_size, &size);
+        ap_send_fd(file, r, sizeof(UploadItem::header), header->file_size, &size);
 
         return OK;
     } catch(const char *message) {
@@ -411,13 +416,11 @@ int remove(request_rec *r, uconfig *config, const char *arg)
 
         try {
             config->write_lock();
-
             config->item_list->remove(file, password);
             config->remove_item_number++;
-
-            config->unlock();
+            config->write_unlock();
         } catch(const char *) {
-            config->unlock();
+            config->write_unlock();
             throw;
         }
 
@@ -444,7 +447,7 @@ int error(request_rec *r, uconfig *config, const char *message)
 
         return OK;
     } catch(const char *message) {
-        ERROR("Exception: %s", message);
+        RERROR("Exception: %s", message);
 
         return HTTP_INTERNAL_SERVER_ERROR;
     }
@@ -470,7 +473,7 @@ int redirect_top(request_rec *r)
 
 uconfig *get_config(request_rec *r)
 {
-    return reinterpret_cast<uconfig *>(ap_get_module_config(r->per_dir_config,
+    return static_cast<uconfig *>(ap_get_module_config(r->per_dir_config,
                                                             &uploader_module));
 }
 
@@ -481,7 +484,8 @@ apr_table_t *get_command_table(request_rec *r)
     apr_table_t *table;
 
     table = apr_table_make(r->pool, MAX_COMMAND_NUMBER);
-    command = reinterpret_cast<char *>(apr_pcalloc(r->pool, MAX_COMMAND_LENGTH));
+
+    APR_PCALLOC(command, char *, r->pool, MAX_COMMAND_LENGTH);
 
     if ((ap_setup_client_block(r, REQUEST_CHUNKED_ERROR) != OK) ||
         !ap_should_client_block(r)) {
@@ -506,10 +510,14 @@ apr_table_t *get_command_table(request_rec *r)
 
 variable *create_variable(apr_pool_t *pool, const char *str)
 {
-    variable *var = reinterpret_cast<variable *>(apr_palloc(pool,
-                                                            sizeof(variable)));
+    variable *var;
+
+    APR_PALLOC(var, variable *, pool, sizeof(variable));
+
     var->type = TemplateExecutor::SCALAR;
-    var->s_val = reinterpret_cast<scalar *>(apr_palloc(pool, sizeof(scalar)));
+
+    APR_PALLOC(var->s_val, scalar *, pool, sizeof(scalar));
+
     var->s_val->type = TemplateExecutor::STRING;
     var->s_val->s_val = str;
 
@@ -518,10 +526,14 @@ variable *create_variable(apr_pool_t *pool, const char *str)
 
 variable *create_variable(apr_pool_t *pool, int i)
 {
-    variable *var = reinterpret_cast<variable *>(apr_palloc(pool,
-                                                            sizeof(variable)));
+    variable *var;
+
+    APR_PALLOC(var, variable *, pool, sizeof(variable));
+
     var->type = TemplateExecutor::SCALAR;
-    var->s_val = reinterpret_cast<scalar *>(apr_palloc(pool, sizeof(scalar)));
+
+    APR_PALLOC(var->s_val, scalar *, pool, sizeof(scalar));
+
     var->s_val->type = TemplateExecutor::INTEGER;
     var->s_val->i_val = i;
 
@@ -545,7 +557,7 @@ variable_map *get_view_vmap(apr_pool_t *pool, UploaderConfig *config,
     ident_id = TemplateExecutor::get_ident_id(tmpl->imap, CURRENT_PAGE_KEY);
     vmap->at(ident_id) = create_variable(pool, static_cast<int>(page));
 
-    page_number = (config->item_list->size() == 0)
+    page_number = config->item_list->empty()
         ? 1
         : (config->item_list->size()-1)/config->per_page_item_number + 1;
     ident_id = TemplateExecutor::get_ident_id(tmpl->imap, PAGE_NUMBER_KEY);
@@ -553,11 +565,11 @@ variable_map *get_view_vmap(apr_pool_t *pool, UploaderConfig *config,
 
     ident_id = TemplateExecutor::get_ident_id(tmpl->imap, TOTAL_FILE_SIZE_KEY);
     vmap->at(ident_id) = create_variable(pool,
-                                         size_str(pool, config->item_list->total_file_size()));
+                                         size_str(pool, config->item_list->get_total_file_size()));
 
     ident_id = TemplateExecutor::get_ident_id(tmpl->imap, TOTAL_FILE_SIZE_KEY);
     vmap->at(ident_id) = create_variable(pool,
-                                         size_str(pool, config->item_list->total_file_size()));
+                                         size_str(pool, config->item_list->get_total_file_size()));
 
      ident_id = TemplateExecutor::get_ident_id(tmpl->imap, TOTAL_FILE_NUMBER_KEY);
      vmap->at(ident_id) = create_variable(pool,
@@ -607,15 +619,15 @@ variable_map *get_thumb_vmap(apr_pool_t *pool, UploaderConfig *config,
     ident_id = TemplateExecutor::get_ident_id(tmpl->imap, CURRENT_PAGE_KEY);
     vmap->at(ident_id) = create_variable(pool, static_cast<int>(page));
 
-    page_number = (config->item_list->thumb_size() == 0)
+    page_number = (config->item_list->get_thumb_size() == 0)
         ? 1
-        : (config->item_list->thumb_size()-1)/config->per_page_item_number + 1;
+        : (config->item_list->get_thumb_size()-1)/config->per_page_item_number + 1;
     ident_id = TemplateExecutor::get_ident_id(tmpl->imap, PAGE_NUMBER_KEY);
     vmap->at(ident_id) = create_variable(pool, static_cast<int>(page_number));
 
     ident_id = TemplateExecutor::get_ident_id(tmpl->imap, THUMB_NUMBER_KEY);
     vmap->at(ident_id) = create_variable(pool,
-                                         comma_str(pool, config->item_list->thumb_size()));
+                                         comma_str(pool, config->item_list->get_thumb_size()));
 
     return vmap;
 }
@@ -663,9 +675,13 @@ static int uploader_handler(request_rec *r)
     config = get_config(r);
 
     try {
-        config->init(r->server->process->pool, r->pool);
+        if (config->init(r->server->process->pool, r->pool)) {
+            config->glock
+                = (static_cast<sconfig *>(ap_get_module_config(r->server->module_config,
+                                                               &uploader_module)))->glock;
+        }
     } catch(const char *message) {
-        ERROR("Exception: %s", message);
+        RERROR("Exception: %s", message);
 
         return HTTP_INTERNAL_SERVER_ERROR;
     }
@@ -677,28 +693,55 @@ static apr_status_t finalize_dir_config(void *data)
 {
     uconfig *config;
 
-    config = reinterpret_cast<uconfig *>(data);
+    config = static_cast<uconfig *>(data);
     config->finalize();
 
     return APR_SUCCESS;
 }
 
-static void *create_dir_config(apr_pool_t *p, char *dirspec)
+static void *create_server_config(apr_pool_t *p, server_rec *s)
+{
+    sconfig *config;
+    const char *tmp_dir;
+
+    config = static_cast<sconfig *>(apr_palloc(p, sizeof(sconfig)));
+
+    apr_temp_dir_get(&tmp_dir, p);
+    apr_filepath_merge(&(config->glock_path), tmp_dir, GLOCK_NAME,
+                       APR_FILEPATH_NOTABOVEROOT, p);
+
+    return config;
+}
+
+static void *create_dir_config(apr_pool_t *pool, char *dirspec)
 {
     uconfig *config;
 
-    config = reinterpret_cast<uconfig *>(apr_pcalloc(p, sizeof(uconfig)));
+    APR_PALLOC(config, uconfig *, pool, sizeof(uconfig));
     new(config) uconfig;
 
-    apr_pool_cleanup_register(p, config, finalize_dir_config, apr_pool_cleanup_null);
+//     config->glock = server_config->glock;
 
-    return reinterpret_cast<uconfig *>(config);
+    apr_pool_cleanup_register(pool, config, finalize_dir_config, apr_pool_cleanup_null);
+
+    return config;
 }
 
 static int post_config(apr_pool_t *pconf, apr_pool_t *plog,
                        apr_pool_t *ptemp, server_rec *s)
 {
-// Windows Ǥ򤹤ȥӥ̾ѤäƤޤ餷
+    sconfig *config;
+
+    config = static_cast<sconfig *>(ap_get_module_config(s->module_config, &uploader_module));
+
+    if (apr_global_mutex_create(&(config->glock), config->glock_path,
+                                APR_LOCK_DEFAULT, pconf) != APR_SUCCESS) {
+        SERROR("Can not create global mutex (%s).", config->glock_path);
+
+        return HTTP_INTERNAL_SERVER_ERROR;
+    }
+
+    // Windows Ǥ򤹤ȥӥ̾ѤäƤޤ餷
 #ifndef WIN32
     ap_add_version_component(pconf, PACKAGE_NAME "/" PACKAGE_VERSION);
 #endif
@@ -706,9 +749,23 @@ static int post_config(apr_pool_t *pconf, apr_pool_t *plog,
     return OK;
 }
 
+static void child_init(apr_pool_t *p, server_rec *s)
+{
+    sconfig *config;
+
+    config = static_cast<sconfig *>(ap_get_module_config(s->module_config, &uploader_module));
+
+    if (apr_global_mutex_child_init(&(config->glock),
+                                    config->glock_path, p) != APR_SUCCESS) {
+        SERROR("Can not attach to global mutex (%s).", config->glock_path);
+
+        return;
+    }
+}
+
 static const char *set_url(cmd_parms *parms, void *mconfig, char *arg)
 {
-    uconfig *config = reinterpret_cast<uconfig *>(mconfig);
+    uconfig *config = static_cast<uconfig *>(mconfig);
 
     config->url = apr_pstrdup(parms->pool, arg);
 
@@ -717,7 +774,7 @@ static const char *set_url(cmd_parms *parms, void *mconfig, char *arg)
 
 static const char *set_file_dir(cmd_parms *parms, void *mconfig, char *arg)
 {
-    uconfig *config = reinterpret_cast<uconfig *>(mconfig);
+    uconfig *config = static_cast<uconfig *>(mconfig);
 
     config->file_dir = apr_pstrdup(parms->pool, arg);
 
@@ -726,7 +783,7 @@ static const char *set_file_dir(cmd_parms *parms, void *mconfig, char *arg)
 
 static const char *set_thumb_dir(cmd_parms *parms, void *mconfig, char *arg)
 {
-    uconfig *config = reinterpret_cast<uconfig *>(mconfig);
+    uconfig *config = static_cast<uconfig *>(mconfig);
 
     config->thumb_dir = apr_pstrdup(parms->pool, arg);
 
@@ -735,7 +792,7 @@ static const char *set_thumb_dir(cmd_parms *parms, void *mconfig, char *arg)
 
 static const char *set_tmp_dir(cmd_parms *parms, void *mconfig, char *arg)
 {
-    uconfig *config = reinterpret_cast<uconfig *>(mconfig);
+    uconfig *config = static_cast<uconfig *>(mconfig);
 
     config->tmp_dir = apr_pstrdup(parms->pool, arg);
 
@@ -744,7 +801,7 @@ static const char *set_tmp_dir(cmd_parms *parms, void *mconfig, char *arg)
 
 static const char *set_lock_dir(cmd_parms *parms, void *mconfig, char *arg)
 {
-    uconfig *config = reinterpret_cast<uconfig *>(mconfig);
+    uconfig *config = static_cast<uconfig *>(mconfig);
 
     config->lock_dir = apr_pstrdup(parms->pool, arg);
 
@@ -753,7 +810,7 @@ static const char *set_lock_dir(cmd_parms *parms, void *mconfig, char *arg)
 
 static const char *set_view_tmpl_path(cmd_parms *parms, void *mconfig, char *arg)
 {
-    uconfig *config = reinterpret_cast<uconfig *>(mconfig);
+    uconfig *config = static_cast<uconfig *>(mconfig);
 
     config->view_tmpl_path = apr_pstrdup(parms->pool, arg);
 
@@ -762,7 +819,7 @@ static const char *set_view_tmpl_path(cmd_parms *parms, void *mconfig, char *arg
 
 static const char *set_download_tmpl_path(cmd_parms *parms, void *mconfig, char *arg)
 {
-    uconfig *config = reinterpret_cast<uconfig *>(mconfig);
+    uconfig *config = static_cast<uconfig *>(mconfig);
 
     config->download_tmpl_path = apr_pstrdup(parms->pool, arg);
 
@@ -771,7 +828,7 @@ static const char *set_download_tmpl_path(cmd_parms *parms, void *mconfig, char
 
 static const char *set_thumb_tmpl_path(cmd_parms *parms, void *mconfig, char *arg)
 {
-    uconfig *config = reinterpret_cast<uconfig *>(mconfig);
+    uconfig *config = static_cast<uconfig *>(mconfig);
 
     config->thumb_tmpl_path = apr_pstrdup(parms->pool, arg);
 
@@ -780,7 +837,7 @@ static const char *set_thumb_tmpl_path(cmd_parms *parms, void *mconfig, char *ar
 
 static const char *set_error_tmpl_path(cmd_parms *parms, void *mconfig, char *arg)
 {
-    uconfig *config = reinterpret_cast<uconfig *>(mconfig);
+    uconfig *config = static_cast<uconfig *>(mconfig);
 
     config->error_tmpl_path = apr_pstrdup(parms->pool, arg);
 
@@ -789,7 +846,7 @@ static const char *set_error_tmpl_path(cmd_parms *parms, void *mconfig, char *ar
 
 static const char *set_max_file_size(cmd_parms *parms, void *mconfig, char *arg)
 {
-    uconfig *config = reinterpret_cast<uconfig *>(mconfig);
+    uconfig *config = static_cast<uconfig *>(mconfig);
 
     config->max_file_size = atoi(arg) * 1024;
 
@@ -798,7 +855,7 @@ static const char *set_max_file_size(cmd_parms *parms, void *mconfig, char *arg)
 
 static const char *set_total_file_size_limit(cmd_parms *parms, void *mconfig, char *arg)
 {
-    uconfig *config = reinterpret_cast<uconfig *>(mconfig);
+    uconfig *config = static_cast<uconfig *>(mconfig);
 
     config->total_file_size_limit = atoi(arg) * 1024;
 
@@ -807,7 +864,7 @@ static const char *set_total_file_size_limit(cmd_parms *parms, void *mconfig, ch
 
 static const char *set_total_file_number_limit(cmd_parms *parms, void *mconfig, char *arg)
 {
-    uconfig *config = reinterpret_cast<uconfig *>(mconfig);
+    uconfig *config = static_cast<uconfig *>(mconfig);
 
     config->total_file_number_limit = atoi(arg);
 
@@ -816,7 +873,7 @@ static const char *set_total_file_number_limit(cmd_parms *parms, void *mconfig,
 
 static const char *set_per_page_item_number(cmd_parms *parms, void *mconfig, char *arg)
 {
-    uconfig *config = reinterpret_cast<uconfig *>(mconfig);
+    uconfig *config = static_cast<uconfig *>(mconfig);
 
     config->per_page_item_number = atoi(arg);
 
@@ -866,9 +923,10 @@ static const command_rec uploader_cmds[] = {
     { NULL }
 };
 
-static void uploader_register_hooks(apr_pool_t *p)
+static void uploader_register_hooks(apr_pool_t *pool)
 {
     ap_hook_post_config(post_config, NULL, NULL, APR_HOOK_REALLY_FIRST);
+    ap_hook_child_init(child_init, NULL, NULL, APR_HOOK_REALLY_FIRST);
     ap_hook_handler(uploader_handler, NULL, NULL, APR_HOOK_MIDDLE);
 }
 
@@ -876,7 +934,7 @@ module AP_MODULE_DECLARE_DATA uploader_module = {
     STANDARD20_MODULE_STUFF,
     create_dir_config,
     NULL,
-    NULL,
+    create_server_config,
     NULL,
     uploader_cmds,
     uploader_register_hooks
